@@ -1,4 +1,4 @@
-const char configNoChecksum[] = "\
+const char *configNoChecksum = "\
 %YAML 1.1\n\
 ---\n\
 pcap-file:\n\
@@ -8,6 +8,7 @@ pcap-file:\n\
 stream:\n\
 \n\
   checksum-validation: no\n\
+  midstream: true\n\
 outputs:\n\
   - fast:\n\
       enabled: yes\n\
@@ -53,7 +54,7 @@ outputs:\n\
         - nfs\n\
         - smb\n\
         - tftp\n\
-        - ikev2\n\
+        - ike\n\
         - krb5\n\
         - snmp\n\
         - rfb\n\
@@ -62,6 +63,7 @@ outputs:\n\
             enabled: yes\n\
             extended: yes\n\
         - ssh\n\
+        - pgsql\n\
         - flow\n\
         - netflow\n\
         - metadata\n\
@@ -73,10 +75,18 @@ outputs:\n\
       enabled: yes\n\
       filename: /dev/null\n\
       extended: yes\n\
+  - file-store:\n\
+      version: 2\n\
+      enabled: yes\n\
+      force-filestore: yes\n\
 app-layer:\n\
   protocols:\n\
     rdp:\n\
       enabled: yes\n\
+    template:\n\
+      enabled: yes\n\
+    template-rust:\n\
+      enabled: yes\n\
     modbus:\n\
       enabled: yes\n\
       detection-ports:\n\
@@ -96,6 +106,10 @@ app-layer:\n\
       hassh: yes\n\
     mqtt:\n\
       enabled: yes\n\
+    pgsql:\n\
+      enabled: yes\n\
     http2:\n\
       enabled: yes\n\
+    quic:\n\
+      enabled: yes\n\
 ";
